/**
 *    Copyright 2011 Peter Murray-Rust et. al.
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

// /*======AUTOGENERATED FROM SCHEMA; DO NOT EDIT BELOW THIS LINE ======*/
package org.xmlcml.cml.element;

import nu.xom.Element;
import org.xmlcml.cml.base.CMLElement;

/** A container for one or more fragments and joins.
*
*
* \n \nfragmentList can contain several fragments and joins. \nThe normal content model is\n \njoin fragment join fragment...\n \n
*
* user-modifiable class autogenerated from schema if no class exists
* use as a shell which can be edited
* the autogeneration software will not overwrite an existing class file

*/
public class CMLFragmentList extends AbstractFragmentList {

	/** namespaced element name.*/
	public final static String NS = C_E+TAG;

    /** constructor
    *
    */
    public CMLFragmentList() {
    }

    /** copy constructor.
    *
    * @param old CMLFragmentList to copy
    */
    public CMLFragmentList(CMLFragmentList old) {
        super((org.xmlcml.cml.element.AbstractFragmentList) old);
    }

    /** copy node .
    *
    * @return Node
    */
    public Element copy() {
        return new CMLFragmentList(this);
    }
    /** create new instance in context of parent, overridable by subclasses.
    *
    * @param parent parent of element to be constructed (ignored by default)
    * @return CMLFragmentList
    */
    public CMLElement makeElementInContext(Element parent) {
        return new CMLFragmentList();
    }

}
