/*    Copyright 2023 Gowri Shankar K
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package org.xmlcml.cml.element.main;

import nu.xom.Element;
import org.junit.Assert;
import org.junit.Test;
import org.xmlcml.cml.base.CMLElement;
import org.xmlcml.cml.element.CMLBandList;

public class CMLBandListTest {

    @Test
    public void testCMLBandList() {
        CMLBandList cmlBandList = new CMLBandList();
        Assert.assertNotNull("constructor: ", cmlBandList);
        cmlBandList.setAttribute("band", "bandList");
        CMLBandList cmlBandList1 = new CMLBandList(cmlBandList);
        Assert.assertEquals("CMLBandList attributeValue should be bandList: ", cmlBandList.getAttributeValue("band"),
                cmlBandList1.getAttributeValue("band"));
    }

    @Test
    public void testCMLBandListCopy() {
        CMLBandList cmlBandList = new CMLBandList();
        cmlBandList.setAttribute("band", "bandList");
        CMLBandList cmlBandList1 = new CMLBandList(cmlBandList);
        Element copy = cmlBandList1.copy();
        Assert.assertEquals("class should be CMLBandList: ", copy.getClass(),
                CMLBandList.class);
    }

    @Test
    public void testCMLBandListMakeElementInContext() {
        Element current = new Element("cmlYaxis");

        CMLBandList factoryElement = new CMLBandList();
        CMLElement newElement = factoryElement.makeElementInContext(current);
        Assert.assertEquals("class should be CMLBandList: ", newElement.getClass(),
                CMLBandList.class);

    }

}
